import typer
from pathlib import Path
from fapi.utils.ustils import render_template, ensure_fastapi_project

add = typer.Typer(help="Adds entities like models, schemas, routes...")

TEMPLATES_DIR = Path("fapi/templates")


@add.command()
def route(name: str):
    """Create a new route file"""
    ensure_fastapi_project()
    if not name:
        typer.echo("You must provide a name for the route")
        raise typer.Exit()

    project_path = Path(".")
    context = {"name": name.lower(), "class_name": name.capitalize()}

    routes_dir = project_path / "app/api/routes"
    routes_dir.mkdir(parents=True, exist_ok=True)

    render_template("app/api/routes/route.py.j2", routes_dir / f"{name.lower()}.py", context)

    typer.echo(f"✅ Route `{name}` created successfully!")


@add.command()
def model(
    name: str,
    schema: bool = typer.Option(False, "--schema", help="Add a schema file"),
    crud: bool = typer.Option(False, "--crud", help="Add a CRUD file")
):
    ensure_fastapi_project()
    if not name:
        typer.echo("You must provide a name for the model")
        raise typer.Exit()

    project_path = Path(".")
    context = {"name": name.lower(), "class_name": name.capitalize()}

    models_dir = project_path / "app/models"
    models_dir.mkdir(parents=True, exist_ok=True)

    render_template("app/models/model.py.j2", models_dir / f"{name.lower()}.py", context)

    if schema:
        schemas_dir = project_path / "app/schemas"
        schemas_dir.mkdir(parents=True, exist_ok=True)

        render_template("app/schemas/schema.py.j2", schemas_dir / f"{name.lower()}_schema.py", context)
        typer.echo(f"📦 Schema `{name}` created")

    if crud:
        crud_dir = project_path / "app/crud"
        crud_dir.mkdir(parents=True, exist_ok=True)

        render_template("app/crud/crud.py.j2", crud_dir / f"{name.lower()}_crud.py", context)
        typer.echo(f"🛠️ CRUD `{name}` created")

    typer.echo(f"✅ Model `{name}` created successfully!")


@add.command()
def service(name: str):
    """Create a new service file"""
    ensure_fastapi_project()

    if not name:
        typer.echo("You must provide a name for the service")
        raise typer.Exit()

    project_path = Path(".")
    context = {"name": name.lower(), "calss_name": name.capitalize()}

    routes_dir = project_path / "app/services"
    routes_dir.mkdir(parents=True, exist_ok=True)

    render_template("app/services/service.py.j2", routes_dir / f"{name.lower()}.py", context)

    typer.echo(f"✅ service `{name}` created successfully!")



@add.command()
def crud(name: str):
    """Create a new crud file"""

    ensure_fastapi_project()

    if not name:
        typer.echo("You must provide a name for the service")
        raise typer.Exit()

    project_path = Path(".")
    context = {"name": name.lower(), "calss_name": name.capitalize()}

    routes_dir = project_path / "app/crud"
    routes_dir.mkdir(parents=True, exist_ok=True)

    render_template("app/crud/crud.py.j2", routes_dir / f"{name.lower()}.py", context)

    typer.echo(f"✅ crud `{name}` created successfully!")

@add.command()
def schema(name: str):
    """Create a new schema file"""
    
    ensure_fastapi_project()

    if not name:
        typer.echo("You must provide a name for the service")
        raise typer.Exit()

    project_path = Path(".")
    context = {"name": name.lower(), "class_name": name.capitalize()}

    routes_dir = project_path / "app/schemas"
    routes_dir.mkdir(parents=True, exist_ok=True)

    render_template("app/schemas/schema.py.j2", routes_dir / f"{name.lower()}.py", context)

    typer.echo(f"✅ schema `{name}` created successfully!")



import typer
from pathlib import Path
from jinja2 import Environment, FileSystemLoader
import subprocess
from fapi.constants import TEMPLATES_DIR


def create_venv(project_dir: Path):
    typer.echo("🛠 Creating virtual environment...")
    subprocess.run(["python3", "-m", "venv", ".venv"], cwd=project_dir)


def req_installer(project_dir: Path):
    req_file = project_dir / "requirements.txt"
    pip_path = project_dir / ".venv" / "bin" / "pip"

    typer.echo("📦 Installing dependencies...")
    subprocess.run([str(pip_path), "install", "-r", str(req_file)])

def render_template(src, dest, context):
    env = Environment(loader=FileSystemLoader(TEMPLATES_DIR))
    template = env.get_template(src)
    rendered = template.render(**context)

    dest.parent.mkdir(parents=True, exist_ok=True)
    dest.write_text(rendered)

def ensure_fastapi_project():
    if not Path(".fastapi").exists():
        typer.echo("❌ This is not a FastAPI project generated by this CLI.")
        typer.echo("Hint: Run `fapi create <project-name>` first.")
        raise typer.Exit(1)



import typer
from fapi.commands.create import create_app
from fapi.commands.add import add

app = typer.Typer(help="FastAPI CLI Tool")
app.add_typer(create_app)
app.add_typer(add, name='add')

if __name__ == "__main__":
    app()



import typer
from fapi.utils.ustils import render_template, req_installer, create_venv
from pathlib import Path
create_app = typer.Typer(help="FastAPI project generator")


@create_app.command()
def create(
    project_name: str = typer.Argument(...),
    db: str = typer.Option(
        None, "--db", help="Database: sqlite or postgres", case_sensitive=False
    ),
    routes: bool = typer.Option(
        None, "--routes", help="Generate routes?"
    ),
):
    """Create a new FastAPI project"""
    project_dir = Path(project_name)

    if project_dir.exists():
        typer.echo(f"Directory {project_name} already exists!")
        raise typer.Exit()

    typer.echo(f"Creating FastAPI project: {project_name}... ✅")
    if db is None:
        use_db = typer.confirm("do you want to use a db?")
        db_choice = typer.prompt("Choose a Database (sqlite/postgres)", default="sqlite") if use_db else None
    else:
        use_db = db.lower() in ["postgres", "sqlite"]
        db_choice = db.lower()
        if not use_db: 
            typer.echo(f"Invalid DB option! {db} neither 'sqlite' not 'postgres")
            raise typer.Exit() 

    if routes is None:
        use_routes = typer.confirm("do you want to generate routes?")
    else:
        use_routes = routes

    

    context = {"project_name": project_name, "use_db":use_db, "db_choice": db_choice, "use_routes": use_routes}

    # Create folders
    (project_dir / "app").mkdir(parents=True, exist_ok=True)

    # Render templates
    render_template("app/main.py.j2", project_dir / "app/main.py", context)
    (project_dir / "app" / "core").mkdir()
    render_template("app/core/config.py.j2", project_dir / "app/core/config.py", context)
    if use_db:
        (project_dir / "app" / "models").mkdir()
        (project_dir / "app" / "schemas").mkdir()
        (project_dir / "app" / "crud").mkdir()
        render_template("app/crud/user.py.j2", project_dir / "app/crud/user.py", context)
        render_template("app/core/database.py.j2", project_dir / "app/core/database.py", context)
        render_template("app/models/user.py.j2", project_dir / "app/models/user.py", context)
        render_template("app/schemas/user.py.j2", project_dir / "app/schemas/user.py", context)
    if use_routes:
        (project_dir / "app" / "api").mkdir()
        (project_dir / "app" / "services").mkdir()
        (project_dir / "app" / "api" / "routes").mkdir()
        render_template("app/api/routes/user.py.j2", project_dir / "app/api/routes/user.py", context)
        render_template("app/api/router.py.j2", project_dir / "app/api/router.py", context)
        render_template("app/services/passwordHash.py.j2", project_dir / "app/services/passwordHash.py", context)
    render_template("requirements.txt.j2", project_dir / "requirements.txt", context)
    render_template("app/__init__.py.j2", project_dir / "app/__init__.py", context)
    # Create .env example
    (project_dir / ".env").write_text("APP_NAME=" + project_name)
    create_venv(project_dir)
    req_installer(project_dir)
    (project_dir / ".fastapi").write_text("fastapi-project")


    typer.echo("✅ Project created successfully!")
    typer.echo(f"Next steps 👉 \n")
    typer.echo(f"cd {project_name}")
    typer.echo("""activate venv: 
               linux / mac: source .venv/bin/activate
                Windows (PowerShell):
                .venv\\Scripts\\Activate.ps1

                Windows (CMD):
                .venv\\Scripts\\activate.bat
               """)
    typer.echo(f"uvicorn app.main:app --reload")
